@@ -25,6 +25,7 @@ from clusterfuzz._internal.build_management import revisions
 from clusterfuzz._internal.chrome import build_info
 from clusterfuzz._internal.datastore import data_handler
 from clusterfuzz._internal.datastore import data_types
+from clusterfuzz._internal.metrics import logs
 from clusterfuzz._internal.system import environment
 
 
@@ -44,6 +45,10 @@ class Impact(object):
     self.likely = likely
     self.extra_trace = extra_trace
 
+  def __str__(self):
+    return (f'version: {self.version}, likely: {self.likely}, '
+            f'extra_trace: {self.extra_trace}')
+
   def is_empty(self):
     """Return True if empty."""
     return not self.version
@@ -104,6 +109,12 @@ def get_start_and_end_revision(regression_range, job_type):
   return start_revision, end_revision
 
 
+def is_valid_regression_range(regression_range, job_type):
+  """Return whether we have a valid regression range."""
+  start, end = get_start_and_end_revision(regression_range, job_type)
+  return start != 0 or end != 0
+
+
 def get_component_information_by_name(chromium_revision,
                                       component_display_name):
   """Returns a dictionary with information about a component at a revision."""
@@ -128,8 +139,12 @@ def get_component_impacts_from_url(component_name,
                                    job_type,
                                    platform=None):
   """Gets component impact string using the build information url."""
+  logs.log('Getting component impacts from URL. Component name %s, '
+           'regression range %s, job type %s, platform %s.' %
+           (component_name, regression_range, str(job_type), str(platform)))
   start_revision, end_revision = get_start_and_end_revision(
       regression_range, job_type)
+  logs.log('Start and end revision %s, %s' % (start_revision, end_revision))
   if not end_revision:
     return Impacts()
 
@@ -140,6 +155,7 @@ def get_component_impacts_from_url(component_name,
   found_impacts = dict()
   for build in ['extended_stable', 'stable', 'beta', 'canary']:
     mapping = build_revision_mappings.get(build)
+    logs.log('Considering impacts for %s.' % (build))
     # TODO(yuanjunh): bypass for now but remove it after ES is enabled.
     if build == 'extended_stable' and not mapping:
       found_impacts[build] = Impact()
@@ -151,18 +167,23 @@ def get_component_impacts_from_url(component_name,
     if not mapping:
       return Impacts()
     chromium_revision = mapping['revision']
+    logs.log('Chromium revision is %s.' % (chromium_revision))
     component_revision = get_component_information_by_name(
         chromium_revision, component_name)
+    logs.log('Component revision is %s.' % (component_revision))
     if not component_revision:
       return Impacts()
     branched_from = revisions.revision_to_branched_from(
         component_revision['url'], component_revision['rev'])
+    logs.log('Branched from revision is %s.' % (branched_from))
     if not branched_from:
-      return Impacts()
+      # This is a head revision, not branched.
+      branched_from = component_revision['rev']
     impact = get_impact({
         'revision': branched_from,
         'version': mapping['version']
     }, start_revision, end_revision, build == 'canary')
+    logs.log('Resulting impact is %s.' % (str(impact)))
     found_impacts[build] = impact
   return Impacts(found_impacts['stable'], found_impacts['beta'],
                  found_impacts['extended_stable'], found_impacts['canary'])
@@ -170,6 +191,8 @@ def get_component_impacts_from_url(component_name,
 
 def get_impacts_from_url(regression_range, job_type, platform=None):
   """Gets impact string using the build information url."""
+  logs.log('Get component impacts from URL: range %s, '
+           'job type %s.' % (regression_range, str(job_type)))
   component_name = data_handler.get_component_name(job_type)
   if component_name:
     return get_component_impacts_from_url(component_name, regression_range,
@@ -177,13 +200,17 @@ def get_impacts_from_url(regression_range, job_type, platform=None):
 
   start_revision, end_revision = get_start_and_end_revision(
       regression_range, job_type)
+  logs.log('Proceeding to calculate impacts as non-component based on '
+           'range %s-%s' % (str(start_revision), str(end_revision)))
   if not end_revision:
     return Impacts()
 
+  logs.log(f'Gathering build to revision mappings for {platform}')
   build_revision_mappings = build_info.get_build_to_revision_mappings(platform)
   if not build_revision_mappings:
     return Impacts()
 
+  logs.log('Calculating impacts from URL')
   extended_stable = get_impact(
       build_revision_mappings.get('extended_stable'), start_revision,
       end_revision)
@@ -360,16 +387,18 @@ def execute_task(testcase_id, job_type):
                                          'Not applicable for custom binaries')
     return
 
+  logs.log('Preparing to calculate impact.')
   # If we don't have a stable or beta build url pattern, we try to use build
   # information url to make a guess.
   if not build_manager.has_production_builds():
-    if not testcase.regression:
+    if not is_valid_regression_range(testcase.regression, testcase.job_type):
       data_handler.update_testcase_comment(
           testcase, data_types.TaskState.FINISHED,
           'Cannot run without regression range, will re-run once regression '
           'task finishes')
       return
 
+    logs.log('No production builds; calculating impact from URL.')
     impacts = get_impacts_from_url(testcase.regression, testcase.job_type)
     testcase = data_handler.get_testcase_by_id(testcase_id)
     set_testcase_with_impacts(testcase, impacts)
@@ -384,6 +413,7 @@ def execute_task(testcase_id, job_type):
 
   # Setup extended stable, stable, beta builds
   # and get impact and crash stacktrace.
+  logs.log('Have production builds; calculating impact by reproduction.')
   try:
     impacts = get_impacts_on_prod_builds(testcase, testcase_file_path)
   except BuildFailedException as error:
