@@ -1094,6 +1094,11 @@ def is_android_kernel(plt=None):
   return 'ANDROID_KERNEL' in (plt or get_platform_group())
 
 
+def is_android_real_device():
+  """Return True if we are on a real android device."""
+  return platform() == 'ANDROID'
+
+
 def is_lib():
   """Whether or not we're in libClusterFuzz."""
   return get_value('LIB_CF')
