@@ -41,7 +41,8 @@ EXISTING_BUCKETS = set(['lib1-logs.clusterfuzz-external.appspot.com'])
 
 def _read_data_file(data_file):
   """Helper function to read the contents of a data file."""
-  with open(os.path.join(DATA_DIRECTORY, data_file)) as handle:
+  with open(
+      os.path.join(DATA_DIRECTORY, data_file), encoding='utf-8') as handle:
     return handle.read()
 
 
@@ -108,7 +109,7 @@ def mock_set_iam_policy(bucket=None, body=None):  # pylint: disable=unused-argum
 def _mock_get_or_create_service_account(project):
   return {
       'email': project + '@serviceaccount.com',
-  }
+  }, False
 
 
 @test_utils.with_cloud_emulators('datastore', 'pubsub')
@@ -116,6 +117,7 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
   """Test project_setup for OSS-Fuzz."""
 
   def setUp(self):
+    self.maxDiff = None  # pylint: disable=invalid-name
     flaskapp = flask.Flask('testflask')
     flaskapp.add_url_rule(
         '/setup', view_func=project_setup.Handler.as_view('/setup'))
@@ -177,6 +179,13 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
     self.honggfuzz.data_bundle_name = 'global'
     self.honggfuzz.put()
 
+    self.gft = data_types.Fuzzer(name='googlefuzztest', jobs=[])
+    self.gft.put()
+
+    self.centipede = data_types.Fuzzer(name='centipede', jobs=[])
+    self.centipede.data_bundle_name = 'global'
+    self.centipede.put()
+
     helpers.patch(self, [
         'clusterfuzz._internal.config.local_config.ProjectConfig',
         ('get_application_id_2',
@@ -202,6 +211,7 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
             'add_revision_mappings': True,
             'build_buckets': {
                 'afl': 'clusterfuzz-builds-afl',
+                'centipede': 'clusterfuzz-builds-centipede',
                 'dataflow': 'clusterfuzz-builds-dataflow',
                 'honggfuzz': 'clusterfuzz-builds-honggfuzz',
                 'libfuzzer': 'clusterfuzz-builds',
@@ -239,6 +249,7 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
                 'User@example.com',
                 'user2@googlemail.com',
             ],
+            'vendor_ccs': None,
         }),
         ('lib2', {
             'homepage': 'http://example2.com',
@@ -289,6 +300,34 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
             'auto_ccs': 'User@example.com',
             'vendor_ccs': ['vendor1@example.com', 'vendor2@example.com'],
         }),
+        ('lib7', {
+            'homepage': 'http://example.com',
+            'primary_contact': 'primary@example.com',
+            'auto_ccs': ['User@example.com',],
+            'fuzzing_engines': ['libfuzzer',],
+            'sanitizers': ['address'],
+            'labels': {
+                '*': ['custom'],
+                'per-target': ['ignore']
+            },
+        }),
+        ('lib8', {
+            'homepage': 'http://example.com',
+            'primary_contact': 'primary@example.com',
+            'auto_ccs': ['User@example.com',],
+            'fuzzing_engines': ['libfuzzer',],
+            'sanitizers': ['none'],
+            'architectures': ['i386', 'x86_64'],
+        }),
+        ('lib9', {
+            'homepage:': 'http://example.com',
+            'primary_contact': 'primary@example.com',
+            'auto_ccs': ['User@example.com',],
+            'main_repo': 'https://github.com/google/main-repo',
+            'fuzzing_engines': ['centipede',],
+            'sanitizers:': ['address',],
+            'architectures': ['x86_64',],
+        }),
     ]
 
     mock_storage.buckets().get.side_effect = mock_bucket_get
@@ -317,7 +356,8 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'CORPUS_BUCKET = lib1-corpus.clusterfuzz-external.appspot.com\n'
         'QUARANTINE_BUCKET = lib1-quarantine.clusterfuzz-external.appspot.com\n'
         'BACKUP_BUCKET = lib1-backup.clusterfuzz-external.appspot.com\n'
-        'AUTOMATIC_LABELS = Proj-lib1,Engine-libfuzzer\n')
+        'AUTOMATIC_LABELS = Proj-lib1,Engine-libfuzzer\n'
+        'FILE_GITHUB_ISSUE = False\n')
 
     job = data_types.Job.query(
         data_types.Job.name == 'libfuzzer_asan_lib2').get()
@@ -343,7 +383,8 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'QUARANTINE_BUCKET = lib3-quarantine.clusterfuzz-external.appspot.com\n'
         'BACKUP_BUCKET = lib3-backup.clusterfuzz-external.appspot.com\n'
         'AUTOMATIC_LABELS = Proj-lib3,Engine-libfuzzer\n'
-        'ISSUE_VIEW_RESTRICTIONS = none\n')
+        'ISSUE_VIEW_RESTRICTIONS = none\n'
+        'FILE_GITHUB_ISSUE = False\n')
 
     job = data_types.Job.query(
         data_types.Job.name == 'libfuzzer_asan_i386_lib3').get()
@@ -364,7 +405,8 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'QUARANTINE_BUCKET = lib3-quarantine.clusterfuzz-external.appspot.com\n'
         'BACKUP_BUCKET = lib3-backup.clusterfuzz-external.appspot.com\n'
         'AUTOMATIC_LABELS = Proj-lib3,Engine-libfuzzer\n'
-        'ISSUE_VIEW_RESTRICTIONS = none\n')
+        'ISSUE_VIEW_RESTRICTIONS = none\n'
+        'FILE_GITHUB_ISSUE = False\n')
 
     job = data_types.Job.query(
         data_types.Job.name == 'libfuzzer_msan_lib3').get()
@@ -386,7 +428,8 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'BACKUP_BUCKET = lib3-backup.clusterfuzz-external.appspot.com\n'
         'AUTOMATIC_LABELS = Proj-lib3,Engine-libfuzzer\n'
         'EXPERIMENTAL = True\n'
-        'ISSUE_VIEW_RESTRICTIONS = none\n')
+        'ISSUE_VIEW_RESTRICTIONS = none\n'
+        'FILE_GITHUB_ISSUE = False\n')
 
     job = data_types.Job.query(
         data_types.Job.name == 'libfuzzer_ubsan_lib3').get()
@@ -407,7 +450,8 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'QUARANTINE_BUCKET = lib3-quarantine.clusterfuzz-external.appspot.com\n'
         'BACKUP_BUCKET = lib3-backup.clusterfuzz-external.appspot.com\n'
         'AUTOMATIC_LABELS = Proj-lib3,Engine-libfuzzer\n'
-        'ISSUE_VIEW_RESTRICTIONS = none\n')
+        'ISSUE_VIEW_RESTRICTIONS = none\n'
+        'FILE_GITHUB_ISSUE = False\n')
 
     job = data_types.Job.query(data_types.Job.name == 'afl_asan_lib1').get()
     self.assertIsNotNone(job)
@@ -427,7 +471,8 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'QUARANTINE_BUCKET = lib1-quarantine.clusterfuzz-external.appspot.com\n'
         'BACKUP_BUCKET = lib1-backup.clusterfuzz-external.appspot.com\n'
         'AUTOMATIC_LABELS = Proj-lib1,Engine-afl\n'
-        'MINIMIZE_JOB_OVERRIDE = libfuzzer_asan_lib1\n')
+        'MINIMIZE_JOB_OVERRIDE = libfuzzer_asan_lib1\n'
+        'FILE_GITHUB_ISSUE = False\n')
 
     # Engine-less job. Manually managed.
     job = data_types.Job.query(data_types.Job.name == 'asan_lib4').get()
@@ -451,8 +496,10 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'BACKUP_BUCKET = lib5-backup.clusterfuzz-external.appspot.com\n'
         'AUTOMATIC_LABELS = Proj-lib5,Engine-libfuzzer\n'
         'EXPERIMENTAL = True\n'
+        'DISABLE_DISCLOSURE = True\n'
         'UNPACK_ALL_FUZZ_TARGETS_AND_FILES = False\n'
-        'MAIN_REPO = https://github.com/google/main-repo\n')
+        'MAIN_REPO = https://github.com/google/main-repo\n'
+        'FILE_GITHUB_ISSUE = False\n')
 
     job = data_types.Job.query(
         data_types.Job.name == 'libfuzzer_asan_lib6').get()
@@ -471,9 +518,56 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'QUARANTINE_BUCKET = lib6-quarantine.clusterfuzz-external.appspot.com\n'
         'BACKUP_BUCKET = lib6-backup.clusterfuzz-external.appspot.com\n'
         'AUTOMATIC_LABELS = Proj-lib6,Engine-libfuzzer\n'
+        'FILE_GITHUB_ISSUE = False\n'
         'DATAFLOW_BUILD_BUCKET_PATH = '
         'gs://clusterfuzz-builds-dataflow/lib6/lib6-dataflow-([0-9]+).zip\n')
 
+    job = data_types.Job.query(
+        data_types.Job.name == 'libfuzzer_asan_lib7').get()
+    self.assertIsNotNone(job)
+    self.assertEqual(job.project, 'lib7')
+    self.assertEqual(job.platform, 'LIB7_LINUX')
+    six.assertCountEqual(self, job.templates,
+                         ['engine_asan', 'libfuzzer', 'prune'])
+    self.assertEqual(
+        job.environment_string, 'RELEASE_BUILD_BUCKET_PATH = '
+        'gs://clusterfuzz-builds/lib7/lib7-address-([0-9]+).zip\n'
+        'PROJECT_NAME = lib7\n'
+        'SUMMARY_PREFIX = lib7\n'
+        'MANAGED = True\n'
+        'REVISION_VARS_URL = https://commondatastorage.googleapis.com/'
+        'clusterfuzz-builds/lib7/lib7-address-%s.srcmap.json\n'
+        'FUZZ_LOGS_BUCKET = lib7-logs.clusterfuzz-external.appspot.com\n'
+        'CORPUS_BUCKET = lib7-corpus.clusterfuzz-external.appspot.com\n'
+        'QUARANTINE_BUCKET = lib7-quarantine.clusterfuzz-external.appspot.com\n'
+        'BACKUP_BUCKET = lib7-backup.clusterfuzz-external.appspot.com\n'
+        'AUTOMATIC_LABELS = Proj-lib7,Engine-libfuzzer,custom\n'
+        'FILE_GITHUB_ISSUE = False\n')
+
+    job = data_types.Job.query(
+        data_types.Job.name == 'centipede_asan_lib9').get()
+    self.assertIsNotNone(job)
+    self.assertEqual(job.project, 'lib9')
+    self.assertEqual(job.platform, 'LIB9_LINUX')
+    six.assertCountEqual(self, job.templates, ['engine_asan', 'centipede'])
+    self.assertEqual(
+        job.environment_string, 'RELEASE_BUILD_BUCKET_PATH = '
+        'gs://clusterfuzz-builds-centipede/lib9/lib9-none-([0-9]+).zip\n'
+        'PROJECT_NAME = lib9\n'
+        'SUMMARY_PREFIX = lib9\n'
+        'MANAGED = True\n'
+        'EXTRA_BUILD_BUCKET_PATH = '
+        'gs://clusterfuzz-builds-centipede/lib9/lib9-address-([0-9]+).zip\n'
+        'REVISION_VARS_URL = https://commondatastorage.googleapis.com/'
+        'clusterfuzz-builds-centipede/lib9/lib9-address-%s.srcmap.json\n'
+        'FUZZ_LOGS_BUCKET = lib9-logs.clusterfuzz-external.appspot.com\n'
+        'CORPUS_BUCKET = lib9-corpus.clusterfuzz-external.appspot.com\n'
+        'QUARANTINE_BUCKET = lib9-quarantine.clusterfuzz-external.appspot.com\n'
+        'BACKUP_BUCKET = lib9-backup.clusterfuzz-external.appspot.com\n'
+        'AUTOMATIC_LABELS = Proj-lib9,Engine-centipede\n'
+        'MAIN_REPO = https://github.com/google/main-repo\n'
+        'FILE_GITHUB_ISSUE = False\n')
+
     self.maxDiff = None  # pylint: disable=invalid-name
 
     libfuzzer = data_types.Fuzzer.query(
@@ -489,6 +583,9 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'libfuzzer_asan_lib6',
         'libfuzzer_msan_lib6',
         'libfuzzer_ubsan_lib6',
+        'libfuzzer_asan_lib7',
+        'libfuzzer_nosanitizer_i386_lib8',
+        'libfuzzer_nosanitizer_lib8',
     ])
 
     afl = data_types.Fuzzer.query(data_types.Fuzzer.name == 'afl').get()
@@ -497,6 +594,12 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'afl_asan_lib6',
     ])
 
+    centipede = data_types.Fuzzer.query(
+        data_types.Fuzzer.name == 'centipede').get()
+    six.assertCountEqual(self, centipede.jobs, [
+        'centipede_asan_lib9',
+    ])
+
     # Test that old unused jobs are deleted.
     self.assertIsNone(
         data_types.Job.query(
@@ -1011,7 +1114,7 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
                     'members': ['serviceAccount:lib1@serviceaccount.com']
                 }]
             },
-            bucket=u'global-corpus.clusterfuzz-external.appspot.com'),
+            bucket='global-corpus.clusterfuzz-external.appspot.com'),
         mock.call(
             body={
                 'resourceId':
@@ -1123,7 +1226,7 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
                     'members': ['serviceAccount:lib2@serviceaccount.com']
                 }]
             },
-            bucket=u'global-corpus.clusterfuzz-external.appspot.com'),
+            bucket='global-corpus.clusterfuzz-external.appspot.com'),
         mock.call(
             body={
                 'resourceId':
@@ -1289,7 +1392,7 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
                     'members': ['serviceAccount:lib3@serviceaccount.com']
                 }]
             },
-            bucket=u'global-corpus.clusterfuzz-external.appspot.com')
+            bucket='global-corpus.clusterfuzz-external.appspot.com')
     ])
 
     mappings = data_types.FuzzerJob.query()
@@ -1308,6 +1411,10 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         ('LIB6_LINUX', 'libFuzzer', 'libfuzzer_ubsan_lib6'),
         ('LIB6_LINUX', 'afl', 'afl_asan_lib6'),
         ('LIB1_LINUX', 'honggfuzz', 'honggfuzz_asan_lib1'),
+        ('LIB7_LINUX', 'libFuzzer', 'libfuzzer_asan_lib7'),
+        ('LIB8_LINUX', 'libFuzzer', 'libfuzzer_nosanitizer_i386_lib8'),
+        ('LIB8_LINUX', 'libFuzzer', 'libfuzzer_nosanitizer_lib8'),
+        ('LIB9_LINUX', 'centipede', 'centipede_asan_lib9'),
     ])
 
     all_permissions = [
@@ -1315,181 +1422,267 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         for entity in data_types.ExternalUserPermission.query()
     ]
 
-    six.assertCountEqual(self, all_permissions, [{
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_asan_lib1',
-        'email': u'primary@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_ubsan_lib1',
-        'email': u'primary@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_ubsan_lib1',
-        'email': u'user@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_ubsan_lib1',
-        'email': u'user2@googlemail.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_asan_lib1',
-        'email': u'user@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_asan_lib1',
-        'email': u'user2@googlemail.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'afl_asan_lib1',
-        'email': u'primary@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'afl_asan_lib1',
-        'email': u'user@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'afl_asan_lib1',
-        'email': u'user2@googlemail.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_msan_lib3',
-        'email': u'user@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_ubsan_lib3',
-        'email': u'user@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'libfuzzer_asan_lib3',
-        'email': u'user@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'auto_cc': 1,
-        'entity_name': u'asan_lib4',
-        'email': u'user@example.com'
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'user@example.com',
-        'entity_name': u'libfuzzer_asan_i386_lib3',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'user@example.com',
-        'entity_name': u'libfuzzer_msan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'user@example.com',
-        'entity_name': u'libfuzzer_ubsan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'user@example.com',
-        'entity_name': u'libfuzzer_asan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'user@example.com',
-        'entity_name': u'afl_asan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor1@example.com',
-        'entity_name': u'libfuzzer_msan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor1@example.com',
-        'entity_name': u'libfuzzer_ubsan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor1@example.com',
-        'entity_name': u'libfuzzer_asan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor1@example.com',
-        'entity_name': u'afl_asan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor2@example.com',
-        'entity_name': u'libfuzzer_msan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor2@example.com',
-        'entity_name': u'libfuzzer_ubsan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor2@example.com',
-        'entity_name': u'libfuzzer_asan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'vendor2@example.com',
-        'entity_name': u'afl_asan_lib6',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'primary@example.com',
-        'entity_name': u'honggfuzz_asan_lib1',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'user@example.com',
-        'entity_name': u'honggfuzz_asan_lib1',
-        'auto_cc': 1
-    }, {
-        'entity_kind': 1,
-        'is_prefix': False,
-        'email': u'user2@googlemail.com',
-        'entity_name': u'honggfuzz_asan_lib1',
-        'auto_cc': 1
-    }])
+    six.assertCountEqual(self, all_permissions, [
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_asan_lib1',
+            'email': 'primary@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_ubsan_lib1',
+            'email': 'primary@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_ubsan_lib1',
+            'email': 'user@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_ubsan_lib1',
+            'email': 'user2@googlemail.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_asan_lib1',
+            'email': 'user@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_asan_lib1',
+            'email': 'user2@googlemail.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'afl_asan_lib1',
+            'email': 'primary@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'afl_asan_lib1',
+            'email': 'user@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'afl_asan_lib1',
+            'email': 'user2@googlemail.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_msan_lib3',
+            'email': 'user@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_ubsan_lib3',
+            'email': 'user@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'libfuzzer_asan_lib3',
+            'email': 'user@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'auto_cc': 1,
+            'entity_name': 'asan_lib4',
+            'email': 'user@example.com'
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'libfuzzer_asan_i386_lib3',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'libfuzzer_msan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'libfuzzer_ubsan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'libfuzzer_asan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'afl_asan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor1@example.com',
+            'entity_name': 'libfuzzer_msan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor1@example.com',
+            'entity_name': 'libfuzzer_ubsan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor1@example.com',
+            'entity_name': 'libfuzzer_asan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor1@example.com',
+            'entity_name': 'afl_asan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor2@example.com',
+            'entity_name': 'libfuzzer_msan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor2@example.com',
+            'entity_name': 'libfuzzer_ubsan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor2@example.com',
+            'entity_name': 'libfuzzer_asan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'vendor2@example.com',
+            'entity_name': 'afl_asan_lib6',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'primary@example.com',
+            'entity_name': 'honggfuzz_asan_lib1',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'honggfuzz_asan_lib1',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user2@googlemail.com',
+            'entity_name': 'honggfuzz_asan_lib1',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'primary@example.com',
+            'entity_name': 'libfuzzer_asan_lib7',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'libfuzzer_asan_lib7',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'libfuzzer_nosanitizer_lib8',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'libfuzzer_nosanitizer_i386_lib8',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'primary@example.com',
+            'entity_name': 'libfuzzer_nosanitizer_lib8',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'primary@example.com',
+            'entity_name': 'libfuzzer_nosanitizer_i386_lib8',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'user@example.com',
+            'entity_name': 'centipede_asan_lib9',
+            'auto_cc': 1
+        },
+        {
+            'entity_kind': 1,
+            'is_prefix': False,
+            'email': 'primary@example.com',
+            'entity_name': 'centipede_asan_lib9',
+            'auto_cc': 1
+        },
+    ])
 
     expected_topics = [
         'projects/clusterfuzz-external/topics/jobs-linux',
@@ -1499,6 +1692,9 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
         'projects/clusterfuzz-external/topics/jobs-lib4-linux',
         'projects/clusterfuzz-external/topics/jobs-lib5-linux',
         'projects/clusterfuzz-external/topics/jobs-lib6-linux',
+        'projects/clusterfuzz-external/topics/jobs-lib7-linux',
+        'projects/clusterfuzz-external/topics/jobs-lib8-linux',
+        'projects/clusterfuzz-external/topics/jobs-lib9-linux',
     ]
     six.assertCountEqual(self, expected_topics,
                          list(pubsub_client.list_topics('projects/' + app_id)))
@@ -1507,7 +1703,7 @@ class OssFuzzProjectSetupTest(unittest.TestCase):
       lib = posixpath.basename(topic).split('-')[1]
       six.assertCountEqual(self, [
           'projects/clusterfuzz-external/subscriptions/'
-          'jobs-{}-linux'.format(lib),
+          f'jobs-{lib}-linux',
       ], pubsub_client.list_topic_subscriptions(topic))
 
     self.assertIsNotNone(pubsub_client.get_topic(unmanaged_topic_name))
@@ -1585,16 +1781,24 @@ def _mock_read_data(path):
               'name':
                   '//a/b',
               'fuzzing_engines': ['libfuzzer', 'honggfuzz'],
-              'sanitizers': ['address']
+              'sanitizers': ['address', 'memory']
           },
           {
               'build_path':
                   'gs://bucket/c-d/%ENGINE%/%SANITIZER%/%TARGET%/([0-9]+).zip',
               'name':
                   '//c/d',
-              'fuzzing_engines': ['libfuzzer'],
+              'fuzzing_engines': ['libfuzzer', 'googlefuzztest'],
               'sanitizers': ['address']
           },
+          {
+              'build_path':
+                  'gs://bucket/e-f/%ENGINE%/%SANITIZER%/%TARGET%/([0-9]+).zip',
+              'name':
+                  '//e/f',
+              'fuzzing_engines': ['libfuzzer'],
+              'sanitizers': ['none']
+          },
       ]
   })
 
@@ -1604,6 +1808,7 @@ class GenericProjectSetupTest(unittest.TestCase):
   """Test generic project setup."""
 
   def setUp(self):
+    self.maxDiff = None  # pylint: disable=invalid-name
     flaskapp = flask.Flask('testflask')
     flaskapp.add_url_rule(
         '/setup', view_func=project_setup.Handler.as_view('/setup'))
@@ -1626,6 +1831,12 @@ class GenericProjectSetupTest(unittest.TestCase):
     self.honggfuzz = data_types.Fuzzer(name='honggfuzz', jobs=[])
     self.honggfuzz.put()
 
+    self.gft = data_types.Fuzzer(name='googlefuzztest', jobs=[])
+    self.gft.put()
+
+    self.centipede = data_types.Fuzzer(name='centipede', jobs=[])
+    self.centipede.put()
+
     helpers.patch(self, [
         'clusterfuzz._internal.config.local_config.ProjectConfig',
         ('get_application_id_2',
@@ -1643,10 +1854,12 @@ class GenericProjectSetupTest(unittest.TestCase):
             {
                 'source': 'gs://bucket/projects.json',
                 'build_type': 'FUZZ_TARGET_BUILD_BUCKET_PATH',
+                'experimental_sanitizers': ['memory'],
                 'build_buckets': {
                     'afl': 'clusterfuzz-builds-afl',
                     'dataflow': 'clusterfuzz-builds-dataflow',
                     'honggfuzz': 'clusterfuzz-builds-honggfuzz',
+                    'googlefuzztest': 'clusterfuzz-builds-googlefuzztest',
                     'libfuzzer': 'clusterfuzz-builds',
                     'libfuzzer_i386': 'clusterfuzz-builds-i386',
                     'no_engine': 'clusterfuzz-builds-no-engine',
@@ -1662,8 +1875,9 @@ class GenericProjectSetupTest(unittest.TestCase):
                             'ASAN_VAR': 'VAL',
                         },
                         'memory': {
-                            'NOT_SET': 'VAL',
-                        }
+                            'MSAN_VAR': 'VAL',
+                        },
+                        'none': {},
                     }
                 }
             },
@@ -1681,6 +1895,7 @@ class GenericProjectSetupTest(unittest.TestCase):
                     'afl': 'clusterfuzz-builds-afl-dbg',
                     'dataflow': 'clusterfuzz-builds-dataflow-dbg',
                     'honggfuzz': 'clusterfuzz-builds-honggfuzz-dbg',
+                    'googlefuzztest': 'clusterfuzz-builds-googlefuzztest-dbg',
                     'libfuzzer': 'clusterfuzz-builds-dbg',
                     'libfuzzer_i386': 'clusterfuzz-builds-i386-dbg',
                     'no_engine': 'clusterfuzz-builds-no-engine-dbg',
@@ -1696,7 +1911,7 @@ class GenericProjectSetupTest(unittest.TestCase):
                             'ASAN_VAR': 'VAL-dbg',
                         },
                         'memory': {
-                            'NOT_SET': 'VAL-dbg',
+                            'MSAN_VAR': 'VAL-dbg',
                         }
                     }
                 }
@@ -1718,6 +1933,8 @@ class GenericProjectSetupTest(unittest.TestCase):
         'FUZZ_TARGET_BUILD_BUCKET_PATH = '
         'gs://bucket/a-b/libfuzzer/address/%TARGET%/([0-9]+).zip\n'
         'PROJECT_NAME = //a/b\nSUMMARY_PREFIX = //a/b\nMANAGED = True\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
         'ASAN_VAR = VAL\n'
         'BOOL_VAR = True\n'
         'INT_VAR = 0\n'
@@ -1728,12 +1945,32 @@ class GenericProjectSetupTest(unittest.TestCase):
     self.assertEqual(None, job.external_updates_subscription)
     self.assertFalse(job.is_external())
 
+    job = data_types.Job.query(
+        data_types.Job.name == 'libfuzzer_msan_a-b').get()
+    self.assertEqual(
+        'FUZZ_TARGET_BUILD_BUCKET_PATH = '
+        'gs://bucket/a-b/libfuzzer/memory/%TARGET%/([0-9]+).zip\n'
+        'PROJECT_NAME = //a/b\nSUMMARY_PREFIX = //a/b\nMANAGED = True\n'
+        'EXPERIMENTAL = True\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
+        'BOOL_VAR = True\n'
+        'INT_VAR = 0\n'
+        'MSAN_VAR = VAL\n'
+        'STRING_VAR = VAL\n', job.environment_string)
+    six.assertCountEqual(self, ['engine_msan', 'libfuzzer'], job.templates)
+    self.assertEqual(None, job.external_reproduction_topic)
+    self.assertEqual(None, job.external_updates_subscription)
+    self.assertFalse(job.is_external())
+
     job = data_types.Job.query(
         data_types.Job.name == 'libfuzzer_asan_c-d').get()
     self.assertEqual(
         'FUZZ_TARGET_BUILD_BUCKET_PATH = '
         'gs://bucket/c-d/libfuzzer/address/%TARGET%/([0-9]+).zip\n'
         'PROJECT_NAME = //c/d\nSUMMARY_PREFIX = //c/d\nMANAGED = True\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
         'ASAN_VAR = VAL\n'
         'BOOL_VAR = True\n'
         'INT_VAR = 0\n'
@@ -1744,6 +1981,22 @@ class GenericProjectSetupTest(unittest.TestCase):
     self.assertEqual(None, job.external_updates_subscription)
     self.assertFalse(job.is_external())
 
+    job = data_types.Job.query(
+        data_types.Job.name == 'libfuzzer_nosanitizer_e-f').get()
+    self.assertEqual(
+        'FUZZ_TARGET_BUILD_BUCKET_PATH = '
+        'gs://bucket/e-f/libfuzzer/none/%TARGET%/([0-9]+).zip\n'
+        'PROJECT_NAME = //e/f\nSUMMARY_PREFIX = //e/f\nMANAGED = True\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
+        'BOOL_VAR = True\n'
+        'INT_VAR = 0\n'
+        'STRING_VAR = VAL\n', job.environment_string)
+    self.assertCountEqual(['libfuzzer', 'prune'], job.templates)
+    self.assertEqual(None, job.external_reproduction_topic)
+    self.assertEqual(None, job.external_updates_subscription)
+    self.assertFalse(job.is_external())
+
     job = data_types.Job.query(
         data_types.Job.name == 'libfuzzer_asan_c-d_dbg').get()
     self.assertIsNone(job)
@@ -1754,6 +2007,8 @@ class GenericProjectSetupTest(unittest.TestCase):
         'FUZZ_TARGET_BUILD_BUCKET_PATH = '
         'gs://bucket-dbg/a-b/libfuzzer/address/%TARGET%/([0-9]+).zip\n'
         'PROJECT_NAME = //a/b\nSUMMARY_PREFIX = //a/b\nMANAGED = True\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
         'ASAN_VAR = VAL-dbg\n'
         'BOOL_VAR = True\n'
         'INT_VAR = 0\n'
@@ -1773,6 +2028,8 @@ class GenericProjectSetupTest(unittest.TestCase):
         'gs://bucket/a-b/honggfuzz/address/%TARGET%/([0-9]+).zip\n'
         'PROJECT_NAME = //a/b\nSUMMARY_PREFIX = //a/b\nMANAGED = True\n'
         'MINIMIZE_JOB_OVERRIDE = libfuzzer_asan_a-b\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
         'BOOL_VAR = True\n'
         'INT_VAR = 0\n'
         'STRING_VAR = VAL\n', job.environment_string)
@@ -1788,6 +2045,8 @@ class GenericProjectSetupTest(unittest.TestCase):
         'gs://bucket-dbg/a-b/honggfuzz/address/%TARGET%/([0-9]+).zip\n'
         'PROJECT_NAME = //a/b\nSUMMARY_PREFIX = //a/b\nMANAGED = True\n'
         'MINIMIZE_JOB_OVERRIDE = libfuzzer_asan_a-b_dbg\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
         'BOOL_VAR = True\n'
         'INT_VAR = 0\n'
         'STRING_VAR = VAL-dbg\n', job.environment_string)
@@ -1798,11 +2057,29 @@ class GenericProjectSetupTest(unittest.TestCase):
                      job.external_updates_subscription)
     self.assertTrue(job.is_external())
 
+    job = data_types.Job.query(
+        data_types.Job.name == 'googlefuzztest_asan_c-d').get()
+    self.assertEqual(
+        'FUZZ_TARGET_BUILD_BUCKET_PATH = '
+        'gs://bucket/c-d/googlefuzztest/address/%TARGET%/([0-9]+).zip\n'
+        'PROJECT_NAME = //c/d\nSUMMARY_PREFIX = //c/d\nMANAGED = True\n'
+        'DISABLE_DISCLOSURE = True\n'
+        'FILE_GITHUB_ISSUE = False\n'
+        'BOOL_VAR = True\n'
+        'INT_VAR = 0\n'
+        'STRING_VAR = VAL\n', job.environment_string)
+    six.assertCountEqual(self, ['engine_asan', 'googlefuzztest'], job.templates)
+    self.assertEqual(None, job.external_reproduction_topic)
+    self.assertEqual(None, job.external_updates_subscription)
+    self.assertFalse(job.is_external())
+
     libfuzzer = data_types.Fuzzer.query(
         data_types.Fuzzer.name == 'libFuzzer').get()
     six.assertCountEqual(self, [
         'libfuzzer_asan_a-b',
         'libfuzzer_asan_c-d',
+        'libfuzzer_msan_a-b',
+        'libfuzzer_nosanitizer_e-f',
         'old_unmanaged',
     ], libfuzzer.jobs)
 
@@ -1814,3 +2091,7 @@ class GenericProjectSetupTest(unittest.TestCase):
     six.assertCountEqual(self, [
         'honggfuzz_asan_a-b',
     ], honggfuzz.jobs)
+
+    gft = data_types.Fuzzer.query(
+        data_types.Fuzzer.name == 'googlefuzztest').get()
+    six.assertCountEqual(self, ['googlefuzztest_asan_c-d'], gft.jobs)
