@@ -76,6 +76,159 @@ CHROMIUM_POLICY = issue_tracker_policy.IssueTrackerPolicy({
     'unreproducible_component': 'Unreproducible>Component'
 })
 
+CHROMIUM_POLICY_FALLBACK = issue_tracker_policy.IssueTrackerPolicy({
+    'status': {
+        'assigned': 'Assigned',
+        'duplicate': 'Duplicate',
+        'verified': 'Verified',
+        'new': 'Untriaged',
+        'wontfix': 'WontFix',
+        'fixed': 'Fixed'
+    },
+    'all': {
+        'status': 'new',
+        'labels': ['ClusterFuzz', 'Stability-%SANITIZER%']
+    },
+    'non_security': {
+        'labels': ['Type-Bug'],
+        'crash_labels': ['Stability-Crash', 'Pri-1'],
+        'non_crash_labels': ['Pri-2']
+    },
+    'labels': {
+        'ignore': 'ClusterFuzz-Ignore',
+        'verified': 'ClusterFuzz-Verified',
+        'security_severity': 'Security_Severity-%SEVERITY%',
+        'needs_feedback': 'Needs-Feedback',
+        'invalid_fuzzer': 'ClusterFuzz-Invalid-Fuzzer',
+        'reported': None,
+        'wrong': 'ClusterFuzz-Wrong',
+        'fuzz_blocker': 'Fuzz-Blocker',
+        'reproducible': 'Reproducible',
+        'auto_cc_from_owners': 'ClusterFuzz-Auto-CC',
+        'os': 'OS-%PLATFORM%',
+        'unreproducible': 'Unreproducible',
+        'restrict_view': 'Restrict-View-SecurityTeam'
+    },
+    'security': {
+        'labels': ['Type-Bug-Security']
+    },
+    'existing': {
+        'labels': ['Stability-%SANITIZER%']
+    },
+    'fallback_component': 'fallback>component',
+    'fallback_policy_message':
+        '**NOTE**: This bug was filed into this component due to permission '
+        'or configuration issues with the specified component(s) %COMPONENTS%'
+})
+
+CHROMIUM_MIRACLEPTR_STACKTRACE_PROTECTED = (
+    """SUMMARY: AddressSanitizer: heap-use-after-free swap.h:36:9
+
+Shadow bytes around the buggy address:
+
+  0x0c1680046160: fa fa f7 fa fd fd fd fd fd fd fd fd fd fd fd fd
+
+  0x0c1680046170: fd fa fa fa fa fa fa fa f7 fa fd fd fd fd fd fd
+
+=>0x0c1680046180: fd fd fd fd fd[fd]fd fa fa fa fa fa fa fa f7 fa
+
+  0x0c1680046190: fd fd fd fd fd fd fd fd fd fd fd fd fd fd fa fa
+
+  0x0c16800461a0: fa fa fa fa f7 fa fd fd fd fd fd fd fd fd fd fd
+
+Shadow byte legend (one shadow byte represents 8 application bytes):
+
+  Addressable:           00
+
+  Partially addressable: 01 02 03 04 05 06 07
+
+  ...
+
+  Right alloca redzone:    cb
+
+
+MiraclePtr Status: PROTECTED
+
+The crash occurred while a raw_ptr<T> object containing a dangling pointer was being dereferenced.
+
+MiraclePtr should make this crash non-exploitable in regular builds.
+
+Refer to https://chromium.googlesource.com/chromium/src/+/main/base/memory/raw_ptr.md for details.
+
+==3196407==ABORTING""")
+
+CHROMIUM_MIRACLEPTR_STACKTRACE_NOT_PROTECTED = (
+    """SUMMARY: AddressSanitizer: heap-use-after-free swap.h:36:9
+
+Shadow bytes around the buggy address:
+
+  0x0c1680046160: fa fa f7 fa fd fd fd fd fd fd fd fd fd fd fd fd
+
+  0x0c1680046170: fd fa fa fa fa fa fa fa f7 fa fd fd fd fd fd fd
+
+=>0x0c1680046180: fd fd fd fd fd[fd]fd fa fa fa fa fa fa fa f7 fa
+
+  0x0c1680046190: fd fd fd fd fd fd fd fd fd fd fd fd fd fd fa fa
+
+  0x0c16800461a0: fa fa fa fa f7 fa fd fd fd fd fd fd fd fd fd fd
+
+Shadow byte legend (one shadow byte represents 8 application bytes):
+
+  Addressable:           00
+
+  Partially addressable: 01 02 03 04 05 06 07
+
+  ...
+
+  Right alloca redzone:    cb
+
+
+MiraclePtr Status: NOT PROTECTED
+
+The crash occurred while a raw_ptr<T> object containing a dangling pointer was being dereferenced.
+
+MiraclePtr should make this crash non-exploitable in regular builds.
+
+Refer to https://chromium.googlesource.com/chromium/src/+/main/base/memory/raw_ptr.md for details.
+
+==3196407==ABORTING""")
+
+CHROMIUM_MIRACLEPTR_STACKTRACE_MANUAL = (
+    """SUMMARY: AddressSanitizer: heap-use-after-free swap.h:36:9
+
+Shadow bytes around the buggy address:
+
+  0x0c1680046160: fa fa f7 fa fd fd fd fd fd fd fd fd fd fd fd fd
+
+  0x0c1680046170: fd fa fa fa fa fa fa fa f7 fa fd fd fd fd fd fd
+
+=>0x0c1680046180: fd fd fd fd fd[fd]fd fa fa fa fa fa fa fa f7 fa
+
+  0x0c1680046190: fd fd fd fd fd fd fd fd fd fd fd fd fd fd fa fa
+
+  0x0c16800461a0: fa fa fa fa f7 fa fd fd fd fd fd fd fd fd fd fd
+
+Shadow byte legend (one shadow byte represents 8 application bytes):
+
+  Addressable:           00
+
+  Partially addressable: 01 02 03 04 05 06 07
+
+  ...
+
+  Right alloca redzone:    cb
+
+
+MiraclePtr Status: MANUAL ANALYSIS REQUIRED
+
+The crash occurred while a raw_ptr<T> object containing a dangling pointer was being dereferenced.
+
+MiraclePtr should make this crash non-exploitable in regular builds.
+
+Refer to https://chromium.googlesource.com/chromium/src/+/main/base/memory/raw_ptr.md for details.
+
+==3196407==ABORTING""")
+
 OSS_FUZZ_POLICY = issue_tracker_policy.IssueTrackerPolicy({
     'status': {
         'assigned': 'Assigned',
@@ -165,24 +318,30 @@ class IssueFilerTests(unittest.TestCase):
   """Tests for the issue filer."""
 
   def setUp(self):
-    data_types.Job(
+    self.job1 = data_types.Job(
         name='job1',
-        environment_string='ISSUE_VIEW_RESTRICTIONS = all\nPROJECT_NAME = proj',
-        platform='linux').put()
+        environment_string=('ISSUE_VIEW_RESTRICTIONS = all\n'
+                            'PROJECT_NAME = proj\n'),
+        platform='linux')
+    self.job1.put()
 
-    data_types.Job(
+    self.job2 = data_types.Job(
         name='job2',
-        environment_string='ISSUE_VIEW_RESTRICTIONS = security',
-        platform='linux').put()
+        environment_string='ISSUE_VIEW_RESTRICTIONS = security\n',
+        platform='linux')
+    self.job2.put()
 
-    data_types.Job(
+    self.job3 = data_types.Job(
         name='job3',
-        environment_string='ISSUE_VIEW_RESTRICTIONS = none',
-        platform='linux').put()
+        environment_string='ISSUE_VIEW_RESTRICTIONS = none\n',
+        platform='linux')
+    self.job3.put()
 
     data_types.Job(
         name='chromeos_job4', environment_string='', platform='linux').put()
 
+    data_types.Job(name='ios_job', environment_string='', platform='mac').put()
+
     testcase_args = {
         'crash_type': 'Heap-use-after-free',
         'crash_address': '0x1337',
@@ -227,6 +386,9 @@ class IssueFilerTests(unittest.TestCase):
         job_type='job', additional_metadata='invalid', **testcase_args)
     self.testcase6.put()
 
+    self.testcase7 = data_types.Testcase(job_type='ios_job4', **testcase_args)
+    self.testcase7.put()
+
     data_types.ExternalUserPermission(
         email='user@example.com',
         entity_name='job2',
@@ -254,12 +416,20 @@ class IssueFilerTests(unittest.TestCase):
     """Tests issue filing for chromium."""
     self.mock.get.return_value = CHROMIUM_POLICY
     issue_tracker = monorail.IssueTracker(IssueTrackerManager('chromium'))
-    issue_filer.file_issue(self.testcase4, issue_tracker)
+    _, exception = issue_filer.file_issue(self.testcase4, issue_tracker)
+    self.assertIsNone(exception)
     self.assertIn('OS-Chrome', issue_tracker._itm.last_issue.labels)
     self.assertEqual('Untriaged', issue_tracker._itm.last_issue.status)
     self.assertNotIn('Restrict-View-SecurityTeam',
                      issue_tracker._itm.last_issue.labels)
 
+  def test_filed_issues_chromium_ios(self):
+    """Tests issue filing for chromium iOS."""
+    self.mock.get.return_value = CHROMIUM_POLICY
+    issue_tracker = monorail.IssueTracker(IssueTrackerManager('chromium'))
+    issue_filer.file_issue(self.testcase7, issue_tracker)
+    self.assertIn('OS-iOS', issue_tracker._itm.last_issue.labels)
+
   def test_filed_issues_chromium_security(self):
     """Tests issue filing for chromium."""
     self.testcase4.security_flag = True
@@ -272,6 +442,26 @@ class IssueFilerTests(unittest.TestCase):
     self.assertIn('Restrict-View-SecurityTeam',
                   issue_tracker._itm.last_issue.labels)
 
+  def test_filed_issues_chromium_miracleptr(self):
+    """Tests MiraclePtr Status label for chromium."""
+    self.mock.get.return_value = CHROMIUM_POLICY
+    helpers.patch(
+        self, ['clusterfuzz._internal.datastore.data_handler.get_stacktrace'])
+    self.mock.get_stacktrace.return_value = CHROMIUM_MIRACLEPTR_STACKTRACE_PROTECTED
+    issue_tracker = monorail.IssueTracker(IssueTrackerManager('chromium'))
+    issue_filer.file_issue(self.testcase1, issue_tracker)
+    self.assertIn('MiraclePtr-Protected', issue_tracker._itm.last_issue.labels)
+
+    self.mock.get_stacktrace.return_value = CHROMIUM_MIRACLEPTR_STACKTRACE_NOT_PROTECTED
+    issue_filer.file_issue(self.testcase1, issue_tracker)
+    self.assertIn('MiraclePtr-NotProtected',
+                  issue_tracker._itm.last_issue.labels)
+
+    self.mock.get_stacktrace.return_value = CHROMIUM_MIRACLEPTR_STACKTRACE_MANUAL
+    issue_filer.file_issue(self.testcase1, issue_tracker)
+    self.assertIn('MiraclePtr-ManualAnalysisRequired',
+                  issue_tracker._itm.last_issue.labels)
+
   def test_filed_issues_oss_fuzz(self):
     """Tests issue filing for oss-fuzz."""
     self.mock.get.return_value = OSS_FUZZ_POLICY
@@ -337,6 +527,24 @@ class IssueFilerTests(unittest.TestCase):
     self.assertIn(FIX_NOTE, issue_tracker._itm.last_issue.body)
     self.assertIn(QUESTIONS_NOTE, issue_tracker._itm.last_issue.body)
 
+  def test_filed_issues_oss_fuzz_disable_disclose(self):
+    """Test filing oss-fuzz issues with disclosure disabled."""
+    self.job2.environment_string += 'DISABLE_DISCLOSURE = True\n'
+    self.job2.put()
+
+    self.mock.get.return_value = OSS_FUZZ_POLICY
+    issue_tracker = monorail.IssueTracker(IssueTrackerManager('oss-fuzz'))
+
+    issue_filer.file_issue(self.testcase2_security, issue_tracker)
+    self.assertTrue(
+        issue_tracker._itm.last_issue.has_label_matching(
+            'restrict-view-commit'))
+    self.assertFalse(
+        issue_tracker._itm.last_issue.has_label_matching('reported-2016-01-01'))
+    self.assertNotIn(DEADLINE_NOTE, issue_tracker._itm.last_issue.body)
+    self.assertIn(FIX_NOTE, issue_tracker._itm.last_issue.body)
+    self.assertIn(QUESTIONS_NOTE, issue_tracker._itm.last_issue.body)
+
   def test_testcase_metadata_labels_and_components(self):
     """Tests issue filing with additional labels and components."""
     self.mock.get.return_value = CHROMIUM_POLICY
@@ -366,6 +574,42 @@ class IssueFilerTests(unittest.TestCase):
         ['ClusterFuzz', 'Reproducible', 'Pri-1', 'Stability-Crash', 'Type-Bug'],
         issue_tracker._itm.last_issue.labels)
 
+  def test_testcase_save_exception(self):
+    """Tests issue filing when issue.save exception"""
+    self.mock.get.return_value = CHROMIUM_POLICY_FALLBACK
+    original_save = monorail.issue.Issue.save
+    helpers.patch(self, ['libs.issue_management.monorail.issue.Issue.save'])
+
+    def my_save(*args, **kwargs):
+      if getattr(my_save, 'raise_exception', True):
+        setattr(my_save, 'raise_exception', False)
+        raise Exception('Boom!')
+      return original_save(*args, **kwargs)
+
+    self.mock.save.side_effect = my_save
+
+    issue_tracker = monorail.IssueTracker(IssueTrackerManager('chromium'))
+    _, exception = issue_filer.file_issue(self.testcase5, issue_tracker)
+    self.assertIsInstance(exception, Exception)
+
+    six.assertCountEqual(self,
+                         ['fallback>component', '-component1', '-component2'],
+                         issue_tracker._itm.last_issue.components)
+    self.assertIn(
+        '**NOTE**: This bug was filed into this component due to permission or '
+        'configuration issues with the specified component(s) component1 component2',
+        issue_tracker._itm.last_issue.body)
+
+    # call without fallback_component in policy
+    # Expected result: no issue is added to itm
+    setattr(my_save, 'raise_exception', True)
+    issue_tracker = monorail.IssueTracker(IssueTrackerManager('chromium'))
+    self.mock.get.return_value = CHROMIUM_POLICY
+    with self.assertRaises(Exception):
+      issue_filer.file_issue(self.testcase1, issue_tracker)
+
+    self.assertIsNone(issue_tracker._itm.last_issue)
+
   @parameterized.parameterized.expand([
       ('chromium', CHROMIUM_POLICY),
       ('oss-fuzz', OSS_FUZZ_POLICY),
@@ -448,7 +692,7 @@ class IssueFilerTests(unittest.TestCase):
         data_types.SecuritySeverity.LOW: 'Security_Severity-Low',
     }
 
-    for security_severity in security_severity_string_map:
+    for security_severity, value in security_severity_string_map.items():
       issue_tracker = monorail.IssueTracker(IssueTrackerManager(project_name))
 
       self.testcase1.security_flag = True
@@ -456,8 +700,7 @@ class IssueFilerTests(unittest.TestCase):
       self.testcase1.put()
 
       issue_filer.file_issue(self.testcase1, issue_tracker)
-      self.assertIn(security_severity_string_map[security_severity],
-                    issue_tracker._itm.last_issue.labels)
+      self.assertIn(value, issue_tracker._itm.last_issue.labels)
       self.assertEqual(
           1,
           len(
@@ -474,7 +717,7 @@ class IssueFilerTests(unittest.TestCase):
     for entry in issue_filer.MEMORY_TOOLS_LABELS:
       issue_tracker = monorail.IssueTracker(IssueTrackerManager(project_name))
 
-      self.testcase1.crash_stacktrace = '\n\n%s\n' % entry['token']
+      self.testcase1.crash_stacktrace = f'\n\n{entry["token"]}\n'
       self.testcase1.put()
       issue_filer.file_issue(self.testcase1, issue_tracker)
       self.assertIn('Stability-' + entry['label'],
@@ -558,7 +801,8 @@ class MemoryToolLabelsTest(unittest.TestCase):
 
   def _read_test_data(self, name):
     """Helper function to read test data."""
-    with open(os.path.join(self.DATA_DIRECTORY, name)) as handle:
+    with open(
+        os.path.join(self.DATA_DIRECTORY, name), encoding='utf-8') as handle:
       return handle.read()
 
   def test_memory_tools_labels_asan(self):
