@@ -572,6 +572,7 @@ class FindMainCrashTest(unittest.TestCase):
     def test_for_repro(fuzzer_name,
                        full_fuzzer_name,
                        file_path,
+                       crash_type,
                        state,
                        security_flag,
                        test_timeout,
@@ -763,8 +764,8 @@ class ProcessCrashesTest(fake_filesystem_unittest.TestCase):
     self.assertEqual(1, len(testcases))
     self.assertEqual('existing', testcases[0].crash_stacktrace)
 
-    variant = data_handler.get_testcase_variant(existing_testcase.key.id(),
-                                                'job')
+    variant = data_handler.get_or_create_testcase_variant(
+        existing_testcase.key.id(), 'job')
     self.assertEqual(data_types.TestcaseVariantStatus.FLAKY, variant.status)
     self.assertEqual('fuzzed_key', variant.reproducer_key)
     self.assertEqual(1234, variant.revision)
@@ -1463,7 +1464,7 @@ class DoEngineFuzzingTest(fake_filesystem_unittest.TestCase):
       self.assertDictEqual({
           'build_revision': 1,
           'command': ['cmd'],
-          'fuzzer': u'libFuzzer_test_target',
+          'fuzzer': 'libFuzzer_test_target',
           'job': 'libfuzzer_asan_test',
           'kind': 'TestcaseRun',
           'stat': 1,
@@ -1517,7 +1518,7 @@ class UntrustedRunEngineFuzzerTest(
     os.makedirs(file_host.rebase_to_worker_root(testcase_directory))
 
     result, fuzzer_metadata, strategies = fuzz_task.run_engine_fuzzer(
-        libfuzzer_engine.LibFuzzerEngine(), 'test_fuzzer', corpus_directory,
+        libfuzzer_engine.Engine(), 'test_fuzzer', corpus_directory,
         testcase_directory)
     self.assertIn(
         'ERROR: AddressSanitizer: SEGV on unknown address 0x000000000000',
@@ -1577,10 +1578,10 @@ class AddIssueMetadataFromEnvironmentTest(unittest.TestCase):
 
   def test_add_numeric(self):
     """Tests adding a numeric label."""
-    os.environ['AUTOMATIC_LABELS'] = '123'
+    os.environ['AUTOMATIC_LABELS'] = '123,456'
 
     metadata = {}
     fuzz_task._add_issue_metadata_from_environment(metadata)
     self.assertDictEqual({
-        'issue_labels': '123',
+        'issue_labels': '123,456',
     }, metadata)
