@@ -44,12 +44,19 @@ class FuzzResult(object):
   """Represents a result of a fuzzing session: a list of crashes found and the
   stats generated."""
 
-  def __init__(self, logs, command, crashes, stats, time_executed):
+  def __init__(self,
+               logs,
+               command,
+               crashes,
+               stats,
+               time_executed,
+               timed_out=None):
     self.logs = logs
     self.command = command
     self.crashes = crashes
     self.stats = stats
     self.time_executed = time_executed
+    self.timed_out = timed_out
 
 
 class ReproduceResult(object):
@@ -202,7 +209,8 @@ def register(name, engine_class):
 
 
 def get(name):
-  """Get an implementation of a fuzzing engine, or None if one does not exist."""
+  """Gets an implementation of a fuzzing engine, or None if one does not
+  exist."""
   engine_class = _ENGINES.get(name)
   if engine_class:
     return engine_class()
