@@ -377,6 +377,18 @@ class StackParser:
           state.lkl_kernel_build_id = match.group(2)
     return result
 
+  @staticmethod
+  def split_stacktrace(stacktrace: str):
+    """Split stacktrace by line, and handle special cases."""
+    # Fix a known malformed traceback pattern:
+    # A newline character is missing between the important crash info line and
+    # an ignorable sanitizer output line.
+    # Insert the newline char back between them so that the crash info can be
+    # preserved for parsing later.
+    stacktrace = re.sub(CONCATENATED_SAN_DEADLYSIGNAL_REGEX,
+                        SPLIT_CONCATENATED_SAN_DEADLYSIGNAL_REGEX, stacktrace)
+    return stacktrace.splitlines()
+
   def parse(self, stacktrace: str) -> CrashInfo:
     """Parse a stacktrace."""
     state = CrashInfo()
@@ -395,7 +407,7 @@ class StackParser:
       stacktrace = self.remove_lkl_kernel_times_and_set_params(
           state, stacktrace)
 
-    split_crash_stacktrace = stacktrace.splitlines()
+    split_crash_stacktrace = StackParser.split_stacktrace(stacktrace)
 
     if state.is_python:
       split_crash_stacktrace = reverse_python_stacktrace(split_crash_stacktrace)
@@ -423,6 +435,7 @@ class StackParser:
         self.match_assert(line, state, ASSERT_REGEX)
         self.match_assert(line, state, ASSERT_REGEX_GOOGLE, group=2)
         self.match_assert(line, state, ASSERT_REGEX_GLIBC)
+        self.match_assert(line, state, ASSERT_REGEX_GLIBC_SUFFIXED)
         self.match_assert(line, state, RUST_ASSERT_REGEX)
 
       # ASSERT_NOT_REACHED prints a single line error then triggers a crash. We
@@ -849,6 +862,10 @@ class StackParser:
           state,
           new_type='Arbitrary DNS resolution')
 
+      # Issue detected by PySecSan
+      self.update_state_on_match(
+          EXTRA_SANITIZERS_PYSECSAN, line, state, new_type='PySecSan')
+
       # For KASan crashes, additional information about a bad access may come
       # from a later line. Update the type and address if this happens.
       update_kasan_crash_details(state, line)
